from chancepy.BaseRandom import BaseRandom
import random
import string
from typing import Optional
import uuid


class Basics(BaseRandom):
    @classmethod
    def guid(cls) -> str:
        return str(uuid.uuid4())

    @classmethod
    def boolean(cls, likelihood: float = 50) -> bool:
        if random.random() * 100 > likelihood:
            return True
        return False

    @classmethod
    def character(cls, pool: Optional[str] = None) -> str:
        randpool = pool
        if not randpool:
            randpool = string.ascii_letters + "!@#$%^&*()1234567890"
        return random.choice(randpool)

    @classmethod
    def letter(cls, casing: str = "lower") -> str:
        pools = {
            "lower": string.ascii_lowercase,
            "upper": string.ascii_uppercase,
        }
        if casing not in pools.keys():
            raise ValueError('Invalid casing. choose "lower" or "upper"')
        return cls.character(pool=pools[casing])

    @classmethod
    def string(cls, length: Optional[int] = None, pool: Optional[str] = None) -> str:
        if not length:
            length = cls.natural(5, 20)

        return "".join(cls.character(pool=pool) for _ in range(length))

    @classmethod
    def floating(cls, mini: float = 0, maxi: float = 100) -> float:
        return random.uniform(mini, maxi)

    @classmethod
    def integer(cls, mini: int = None, maxi: int = None) -> int:
        print(mini, maxi)
        return random.randint(
            mini if mini is not None else cls.MIN_INT, maxi or cls.MAX_INT
        )

    @classmethod
    def natural(cls, mini: int = 0, maxi: int = None) -> int:
        mini = max(0, mini)
        return cls.integer(mini, maxi or cls.MAX_INT)



import pytz
from .BaseRandom import BaseRandom
import datetime


class TimeChance(BaseRandom):
    @classmethod
    def ampm(cls):
        return cls.random.choice(["am", "pm"])

    @classmethod
    def date(cls, year=None, month=None, day=None):
        # this method might fail if 29 of februray is selected
        random_date = datetime.date.fromtimestamp(cls.timestamp())
        match = False
        while not match:
            match_month = random_date.month == month if month is not None else True
            match_day = random_date.day == day if day is not None else True

            match = match_month and match_day
            if not match:
                random_date = random_date + datetime.timedelta(1)
        if year:
            random_date = datetime.date(
                year=year, month=random_date.month, day=random_date.day
            )
        return random_date

    @classmethod
    def timestamp(cls):
        now = int(datetime.datetime.now().timestamp())
        random_timestamp_until_now = cls.random.randint(0, now)
        return random_timestamp_until_now

    @classmethod
    def year(cls, mini=None, maxi=None):
        if not mini:
            mini = datetime.date.today().year
        return cls.random.randint(mini, maxi if maxi else mini + 100)

    @classmethod
    def month(cls, mode="numeric"):
        month = cls.random.randint(1, 12)
        if mode == "numeric":
            return month
        if mode == "short":
            return datetime.date(1970, month, 0).strftime("%b")
        if mode == "full":
            return datetime.date(1970, month, 1).strftime("%B")
        raise ValueError("invalid mode, choose 'numeric', 'short' or 'full")

    @classmethod
    def weekday(cls, mode="full"):
        date = cls.date()
        if mode == "full":
            return date.strftime("%A")
        if mode == "short":
            return date.strftime("%a")
        if mode == "numeric":
            return date.weekday()
        raise ValueError("invalid mode, choose 'numeric', 'short' or 'full")

    @classmethod
    def hour(cls, twentyfour=False):
        return cls.random.randint(1, 24 if twentyfour else 12)

    @classmethod
    def minute(cls):
        return cls.random.randint(0, 59)

    @classmethod
    def second(cls):
        return cls.random.randint(0, 59)

    @classmethod
    def millisecond(cls):
        return cls.random.randint(0, 999)

    @classmethod
    def timezone(cls):
        return pytz.timezone(cls.random.choice(pytz.all_timezones))



from chancepy.BaseRandom import BaseRandom
import random
from typing import List


class Utils(BaseRandom):
    @classmethod
    def pickone(cls, array: List):
        return random.choice(array)

    @classmethod
    def pickset(cls, array: List, k: int):
        return cls.random.sample(array, k)
